#ifndef RESCUE_INPUT_H
#define RESCUE_INPUT_H

#include "common.h"
#include <string>

namespace Librescue
{
	class Overrun {
	public:
	  Overrun();
	  ~Overrun();
	  std::string message;
	};

	class Input {
	protected:
		Bytes m_bytes;
		int m_index;

	public:
		Input();	
		~Input();
		
		void reset(const Bytes &source);
		
		// Get the total size of this buffer
		int size() const;

		Cursor cursor() const;
		void setCursor(Cursor cursor);
		void skip(INT_32 numBytes);

		INT_32 readInt32();
		INT_32 peekInt32();
		// Read some number of bytes into a buffer
		void read(int size, Bytes& buffer);
		void read(int size, Byte* buffer);
		// Read a string into a char array. The size will be read first.
		void readString(char* result); // Better hope that result is long enough!
		// Read a string of a particular size
		void readString(char* result, int size);
		// Read a string and return a new string object. The size will be read first.
		std::string readString();
	};

	// Read all RescueObjects described in a buffer and store the result in an Objects buffer. This will fill "result" with new RescueObjects - don't forget to delete them later!
	void readObjects(Input& input, int time, Objects& result);
	class RescueObject;
	// Allocate and populate a new RescueObject. This function will return a new RescueObject. Don't forget to delete it later!
	RescueObject* readObject(Input& input, int time);

} //namespace rescue;

#endif // !defined ...
