#ifndef RESCUE_OBJECTPOOL_H
#define RESCUE_OBJECTPOOL_H

#include "objects.h"
#include "common.h"
#include "output.h"
#include <map>

namespace Librescue {
  class ObjectPool {
  private:
	typedef std::map<Id,RescueObject*> IdToRescueObject;
	IdToRescueObject data;
	Objects allData;

  public:
	ObjectPool();
	// This will delete all RescueObjects that we know about.
	virtual ~ObjectPool();

	// Add an object to the pool
	void addObject(RescueObject* object);
	// Remove an object from the pool
	void removeObject(RescueObject* object);
	// Find an object by ID
	RescueObject* getObject(Id id);
	const RescueObject* getObject(Id id) const;
	// Get the neighbours of an object
	Objects getNeighbours(Id object);
	Objects getNeighbours(const RescueObject* object);
	// Get all objects
	const Objects& objects() const;
	// Find all objects with a certain radius of a point and store them in "result". Returns the number of objects found.
	int getObjectsInRange(int x, int y, int distance,std::list<RescueObject*>& result) const;
	// Find the distance between two objects. Returns -1 if the range could not be calculated.
	int range(const RescueObject* o1, const RescueObject* o2) const;
	int range(Id o1, Id o2) const;
	int range(const RescueObject* o1, Id o2) const;
	int range(Id o1, const RescueObject* o2) const;
	int range(int x1, int y1, int x2, int y2) const;
	// Find the coordinates of an object and store the result in x and y. Returns true iff the object could be located.
	bool locate(const RescueObject*, int* x, int* y) const;
	// Write all objects to an Output
	void write(Output& out) const;
	// Write all objects that have changed after a particular time to an Output
	void write(Output& out, int time) const;

	// Update from a buffer
	void update(Input& buffer, int time);
  };
}

#endif
