#include "output.h"
#include "objects.h"
#include "error.h"

namespace Librescue {
  void writeGK_CONNECT_OK(Output& output, Objects objects) {
	Cursor base = output.writeInt32(GK_CONNECT_OK);
	Cursor start = output.writeInt32(0);
	int n = objects.size();
	for (int i=0;i<n;++i) {
	  objects[i]->write(output,0);
	}
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeGK_CONNECT_ERROR(Output& output, char* reason) {
	Cursor base = output.writeInt32(GK_CONNECT_ERROR);
	Cursor start = output.writeInt32(0);
	output.writeString(reason);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeGK_CONNECT_ERROR(Output& output, std::string reason) {
	Cursor base = output.writeInt32(GK_CONNECT_ERROR);
	Cursor start = output.writeInt32(0);
	output.writeString(reason);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKG_CONNECT(Output& output, INT_32 version) {
	Cursor base = output.writeInt32(KG_CONNECT);
	Cursor start = output.writeInt32(0);
	output.writeInt32(version);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKG_ACKNOWLEDGE(Output& output) {
	output.writeInt32(KG_ACKNOWLEDGE);
	output.writeInt32(0);
  }

  void writeKG_UPDATE(Output& output, INT_32 time, Objects update) {
	Cursor base = output.writeInt32(KG_UPDATE);
	Cursor start = output.writeInt32(0);
	output.writeInt32(time);
	int n = update.size();
	for (int i=0;i<n;++i) {
	  update[i]->write(output,time);
	}
	output.writeInt32(TYPE_NULL);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeSK_CONNECT(Output& output, INT_32 version) {
	Cursor base = output.writeInt32(SK_CONNECT);
	Cursor start = output.writeInt32(0);
	output.writeInt32(version);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeSK_ACKNOWLEDGE(Output& output) {
	output.writeInt32(SK_ACKNOWLEDGE);
	output.writeInt32(0);
  }

  void writeSK_UPDATE(Output& output, Objects update) {
	Cursor base = output.writeInt32(SK_UPDATE);
	Cursor start = output.writeInt32(0);
	int n = update.size();
	for (int i=0;i<n;++i) {
	  update[i]->write(output,0);
	}
	output.writeInt32(TYPE_NULL);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKS_CONNECT_OK(Output& output, Objects objects) {
	Cursor base = output.writeInt32(KS_CONNECT_OK);
	Cursor start = output.writeInt32(0);
	int n = objects.size();
	for (int i=0;i<n;++i) {
	  objects[i]->write(output,0);
	}
	output.writeInt32(TYPE_NULL);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKS_CONNECT_ERROR(Output& output, char* reason) {
	Cursor base = output.writeInt32(KS_CONNECT_ERROR);
	Cursor start = output.writeInt32(0);
	output.writeString(reason);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKS_CONNECT_ERROR(Output& output, std::string reason) {
	Cursor base = output.writeInt32(KS_CONNECT_ERROR);
	Cursor start = output.writeInt32(0);
	output.writeString(reason);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKS_COMMANDS(Output& output, INT_32 time, Commands commands);

  void writeKS_UPDATE(Output& output, INT_32 time, Objects update) {
	Cursor base = output.writeInt32(KS_UPDATE);
	Cursor start = output.writeInt32(0);
	output.writeInt32(time);
	int n = update.size();
	for (int i=0;i<n;++i) {
	  update[i]->write(output,time);
	}
	output.writeInt32(TYPE_NULL);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeVK_CONNECT(Output& output, INT_32 version) {
	Cursor base = output.writeInt32(VK_CONNECT);
	Cursor start = output.writeInt32(0);
	output.writeInt32(version);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeVK_ACKNOWLEDGE(Output& output) {
	output.writeInt32(VK_ACKNOWLEDGE);
	output.writeInt32(0);
  }

  void writeKV_CONNECT_OK(Output& output, Objects objects) {
	Cursor base = output.writeInt32(KV_CONNECT_OK);
	Cursor start = output.writeInt32(0);
	int n = objects.size();
	for (int i=0;i<n;++i) {
	  objects[i]->write(output,0);
	}
	output.writeInt32(TYPE_NULL);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKV_CONNECT_ERROR(Output& output, char* reason) {
	Cursor base = output.writeInt32(VK_CONNECT);
	Cursor start = output.writeInt32(0);
	output.writeString(reason);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKV_CONNECT_ERROR(Output& output, std::string reason) {
	Cursor base = output.writeInt32(VK_CONNECT);
	Cursor start = output.writeInt32(0);
	output.writeString(reason);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKV_UPDATE(Output& output, INT_32 time, Objects update) {
	Cursor base = output.writeInt32(KV_UPDATE);
	Cursor start = output.writeInt32(0);
	output.writeInt32(time);
	int n = update.size();
	for (int i=0;i<n;++i) {
	  update[i]->write(output,time);
	}
	output.writeInt32(TYPE_NULL);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_CONNECT(Output& output, INT_32 version, INT_32 tempID, AgentType type) {
	Cursor base = output.writeInt32(AK_CONNECT);
	Cursor start = output.writeInt32(0);
	output.writeInt32(tempID);
	output.writeInt32(version);
	output.writeInt32(type);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_ACKNOWLEDGE(Output& output, Id id) {
	Cursor base = output.writeInt32(AK_ACKNOWLEDGE);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKA_CONNECT_OK(Output& output, INT_32 tempID, Id id, RescueObject& self, Objects knowledge) {
	Cursor base = output.writeInt32(KA_CONNECT_OK);
	Cursor start = output.writeInt32(0);
	output.writeInt32(tempID);
	output.writeInt32(id);
	self.write(output,0);
	int n = knowledge.size();
	for (int i=0;i<n;++i) {
	  knowledge[i]->write(output,0);
	}
	output.writeInt32(TYPE_NULL);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKA_CONNECT_ERROR(Output& output, char* reason) {
	Cursor base = output.writeInt32(KA_CONNECT_ERROR);
	Cursor start = output.writeInt32(0);
	output.writeString(reason);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKA_CONNECT_ERROR(Output& output, std::string reason) {
	Cursor base = output.writeInt32(KA_CONNECT_ERROR);
	Cursor start = output.writeInt32(0);
	output.writeString(reason);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKA_SENSE(Output& output, Id id, INT_32 time, RescueObject& self, Objects update) {
	Cursor base = output.writeInt32(KA_SENSE);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	output.writeInt32(time);
	self.write(output,time);
	int n = update.size();
	for (int i=0;i<n;++i) {
	  update[i]->write(output,time);
	}
	output.writeInt32(TYPE_NULL);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKA_HEAR(Output& output, Id to, Id from, INT_32 size, char* data) {
	Cursor base = output.writeInt32(KA_HEAR);
	Cursor start = output.writeInt32(0);
	output.writeString(data,size);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKA_HEAR_SAY(Output& output, Id to, Id from, INT_32 size, char* data) {
	Cursor base = output.writeInt32(KA_HEAR_SAY);
	Cursor start = output.writeInt32(0);
	output.writeString(data,size);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeKA_HEAR_TELL(Output& output, Id to, Id from, INT_32 size, char* data) {
	Cursor base = output.writeInt32(KA_HEAR_TELL);
	Cursor start = output.writeInt32(0);
	output.writeString(data,size);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_REST(Output& output, Id id) {
	Cursor base = output.writeInt32(AK_REST);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_MOVE(Output& output, Id id, Ids route) {
	Cursor base = output.writeInt32(AK_MOVE);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	int n = route.size();
	for (int i=0;i<n;++i) {
	  output.writeInt32(route[i]);
	}
	output.writeInt32(0);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_SAY(Output& output, char* message, INT_32 length, Id sender) {
	Cursor base = output.writeInt32(AK_SAY);
	Cursor start = output.writeInt32(0);
	output.writeInt32(sender);
	output.writeString(message,length);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_TELL(Output& output, char* message, INT_32 length, Id sender) {
	Cursor base = output.writeInt32(AK_TELL);
	Cursor start = output.writeInt32(0);
	output.writeInt32(sender);
	output.writeString(message,length);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_EXTINGUISH(Output& output, Id id, Id target, INT_32 direction, INT_32 nozzleX, INT_32 nozzleY, INT_32 water) {
	Cursor base = output.writeInt32(AK_EXTINGUISH);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	output.writeInt32(target);
	output.writeInt32(direction);
	output.writeInt32(nozzleX);
	output.writeInt32(nozzleY);
	output.writeInt32(water);
	output.writeInt32(0);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_EXTINGUISH(Output& output, Id id, Id target, INT_32 water) {
	writeAK_EXTINGUISH(output,id,target,0,0,0,water);
  }

  void writeAK_RESCUE(Output& output, Id id, Id target) {
	Cursor base = output.writeInt32(AK_RESCUE);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	output.writeInt32(target);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_LOAD(Output& output, Id id, Id target) {
	Cursor base = output.writeInt32(AK_LOAD);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	output.writeInt32(target);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_UNLOAD(Output& output, Id id) {
	Cursor base = output.writeInt32(AK_UNLOAD);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  void writeAK_CLEAR(Output& output, Id id, Id target) {
	Cursor base = output.writeInt32(AK_CLEAR);
	Cursor start = output.writeInt32(0);
	output.writeInt32(id);
	output.writeInt32(target);
	Cursor end = output.cursor();
	output.writeInt32(end-start, base);
	output.setCursor(end);
  }

  Output::Output() {
	m_bytes.clear();
	m_index = 0;
  }

  Output::~Output() {}
		
  void Output::clear() {
	m_bytes.clear();
	m_index = 0;
  }

  Cursor Output::cursor() const {
	return m_index;
  }

  void Output::setCursor(Cursor cursor) {
	m_index = cursor;
  }

  Cursor Output::writeInt32(INT_32 value) {
	m_bytes.push_back((value>>24)&0xFF);
	m_bytes.push_back((value>>16)&0xFF);
	m_bytes.push_back((value>>8)&0xFF);
	m_bytes.push_back(value&0xFF);
	m_index += 4;
	return m_index;
  }

  Cursor Output::writeInt32(INT_32 value, Cursor position) {
	// First make sure that m_bytes is big enough
	m_bytes.reserve(position+4);
	m_bytes[position] = ((value>>24)&0xFF);
	m_bytes[position+1] = ((value>>16)&0xFF);
	m_bytes[position+2] = ((value>>8)&0xFF);
	m_bytes[position+3] = (value&0xFF);
	return position+4;
  }

  Cursor Output::writeString(const char* string) {
	int length = strlen(string);
	return writeString(string,length+1); // Use length+1 to make sure that the null byte at the end gets written
  }

  Cursor Output::writeString(std::string string) {
	return writeString(string.c_str(),string.length()+1); // Use length+1 to make sure that the null byte at the end gets written
  }

  Cursor Output::writeString(const char* string, INT_32 size) {
	writeInt32(size);
	for (int i=0;i<size;++i) {
	  m_bytes.push_back(string[i]);
	  ++m_index;
	}
	return m_index;	
  }

  Cursor Output::write(int size, Bytes& buffer) {
	for (int i=0;i<size;++i) {
	  m_bytes.push_back(buffer[i]);
	}
	m_index += size;
	return m_index;
  }

  Cursor Output::write(int size, Byte* buffer) {
	for (int i=0;i<size;++i) {
	  m_bytes.push_back(buffer[i]);
	}
	m_index += size;
	return m_index;
  }

  Cursor Output::write(Bytes& buffer) {
	m_bytes.insert(m_bytes.end(),buffer.begin(),buffer.end());
	m_index += buffer.size();
	return m_index;
  }

  Cursor Output::write(Output& out) {
	return write(out.m_bytes);
  }

  Cursor Output::writeSize(Cursor base) {
	Cursor current = cursor();
	writeInt32(current-base,base-INT_32_SIZE);
	setCursor(current);
	return current;
  }

  const Bytes& Output::buffer() const {
	return m_bytes;
  }

  int Output::size() const {
	return m_index;
  }

  void Output::log(FILE* file) const {
	long s = m_bytes.size();
	//	LOG_DEBUG("Writing output to file. Size=%d",(int)s);
	Byte b[4];
	b[0] = (Byte)((s >> 24) & 0xFF);
	b[1] = (Byte)((s >> 16) & 0xFF);
	b[2] = (Byte)((s >> 8) & 0xFF);
	b[3] = (Byte)(s & 0xFF);
	fwrite(b, sizeof(Byte), 4, file);
	long written = 0;
	while (written < s) {
	  written += fwrite(&m_bytes[written], sizeof(Byte), s-written, file);
	}
	fflush(file);
  }
}
