#ifndef RESCUE_UDP_H
#define RESCUE_UDP_H

#include "connection.h"
#include "common.h"
#include "input.h"
#include "output.h"
#include <string>
#include <netinet/in.h>
#include <time.h>

#define LONGUDP_DEFAULT_PACKET_SIZE 1024
// Default timeout is 60 seconds
#define LONGUDP_DEFAULT_TIMEOUT 60
// Default wait between packets is 1ms
#define LONGUDP_DEFAULT_WAIT 1

namespace Librescue {
  class LongUDPConnection : public Connection {
  private:
	struct Fragment {
	  INT_16 id;
	  INT_16 index;
	  INT_16 total;
	  Byte* data;
	  int size;

	  Fragment() {
		data = 0;
	  }

	  void fill(INT_16 id_, INT_16 index_, INT_16 total_, Byte* buffer_, int size_) {
		id = id_;
		index = index_;
		total = total_;
		data = new Byte[size_];
		memcpy(data,buffer_,size_);
		size = size_;
	  }

	  ~Fragment() {
		if (data) delete[] data;
	  }
	};

	struct PartialPacket {
	  Fragment* fragments;
	  int totalFragments;
	  int received;
	  time_t lastTime;
	  INT_16 id;
	  Address from;
	  int totalSize;

	  PartialPacket(INT_16 id_, int total, Address from_) : from(from_) {
		id = id_;
		totalFragments = total;
		fragments = new Fragment[total];
		received = 0;
		totalSize = 0;
	  }

	  ~PartialPacket() {
		delete[] fragments;
	  }
	};

	typedef std::list<PartialPacket*> PartialPackets;
	PartialPackets partialPackets;
	std::list<Bytes*> finishedPackets;
	std::list<Address> finishedFrom;
	int m_packetSize;
	time_t m_timeout;
	int m_wait;
	int m_socket;
	Address m_target;
	Address m_from;
	int m_nextID;

	// Reads packets from the socket and processes them until there is nothing left to read.
	void readUDP(int timeout);

	void init(const Address& localAddress, int packetSize, int wait, time_t timeout);

  public:
	//	LongUDPConnection(Address& localAddress);
	//	LongUDPConnection(Address& localAddress, int packetSize);
	LongUDPConnection(const char* localHost = "localhost", int localPort = 0, int packetSize = LONGUDP_DEFAULT_PACKET_SIZE, int wait = LONGUDP_DEFAULT_WAIT, time_t timeout = LONGUDP_DEFAULT_TIMEOUT);
	LongUDPConnection(const Address& localAddress, int packetSize = LONGUDP_DEFAULT_PACKET_SIZE, int wait = LONGUDP_DEFAULT_WAIT, time_t timeout = LONGUDP_DEFAULT_TIMEOUT);
	virtual ~LongUDPConnection();

	virtual bool send(const Output& buffer, const Address& address);
	virtual bool send(const Output& buffer);
	virtual void receive(Input& buffer);
	virtual bool isDataAvailable(int timeout);
	// Get the Address that the last message was received from
	virtual const Address& addressReceivedFrom() const;

	virtual void close();
	virtual bool isOpen();

	void setPacketSize(int size);
	void setWait(int wait);
	void setTimeout(int timeout);
	void setTarget(const Address& address);
  };
}

#endif
