package viewer;

import java.io.*;
import java.net.*;
import java.util.*;

public class TCPIO extends IO {
	private Socket socket;
	private DataInputStream in;
	private DataOutputStream out;

	public TCPIO(InetAddress kernelAddress, int kernelPort) {
		try{
			socket  = new Socket(kernelAddress, kernelPort);
			in = new DataInputStream(socket.getInputStream());
			out = new DataOutputStream(socket.getOutputStream());
		} catch (Exception e) { e.printStackTrace();  System.exit(1); }
	}

	protected byte[] receive() {
		try {
			int byte0 = in.read();
			int byte1 = in.read();
			int byte2 = in.read();
			int byte3 = in.read();
			int length = byte0<<24 | byte1<<16 | byte2<<8 | byte3;
			//			System.out.println("Received: "+Integer.toHexString(byte0)+" "+Integer.toHexString(byte1)+" "+Integer.toHexString(byte2)+" "+Integer.toHexString(byte3));
			//			System.out.println("Length = "+length);
			int total = 0;
			byte[] result = new byte[length];
			while (total < length) {
				total += in.read(result,total,length-total);
				//				System.out.println("Read "+total+" of "+length+" bytes");
			}
			return result;
		} catch (Exception e) { e.printStackTrace();  System.exit(1); }
		return null;
	}

	protected void send(int header, byte[] body) {
		try {
			int len = body.length + 12;
			//			out.writeInt(len);
			int byte0 = (len>>24) & 0xFF;
			int byte1 = (len>>16) & 0xFF;
			int byte2 = (len>>8) & 0xFF;
			int byte3 = len & 0xFF;
			//			System.out.println("dataToSend length: "+len);
			//			System.out.println(Integer.toHexString(byte0)+" "+Integer.toHexString(byte1)+" "+Integer.toHexString(byte2)+" "+Integer.toHexString(byte3));
			out.writeByte((byte)byte0);
			out.writeByte((byte)byte1);
			out.writeByte((byte)byte2);
			out.writeByte((byte)byte3);
			out.writeInt(header);
			out.writeInt(body.length);
			out.write(body);
			out.writeInt(HEADER_NULL);
			out.flush();
			//			System.out.println("sending complete");
		} catch (Exception e) { e.printStackTrace();  System.exit(1); }
	}

	public void close() {
		try
			{
				in.close();
				out.close();
				socket.close();
			}
		catch (IOException e)
			{
				e.printStackTrace(); System.exit(1);
			}
	}

	/*
	private class LoggedOutputStream extends OutputStream {
		private OutputStream down;
		private StringBuffer buffer;
		private int count;

		public LoggedOutputStream(OutputStream down) {
			this.down = down;
			buffer = new StringBuffer();
			count = 0;
		}

		public void write(int i) throws IOException {
			buffer.append(Integer.toHexString(i));
			buffer.append(" ");
			++count;
			if (count==4) {
				System.out.println(buffer.toString());
				buffer.setLength(0);
				count = 0;
			}
			down.write(i);
		}

		public void flush() throws IOException {
			if (count!=0) {
				System.out.println(buffer.toString());
				buffer.setLength(0);
				count = 0;
			}
			System.out.println("Flushed");
			down.flush();
		}

		public void close() throws IOException {
			if (count!=0) {
				System.out.println(buffer.toString());
				buffer.setLength(0);
				count = 0;
			}
			System.out.println("Flushed");
			down.close();
		}
	}
	*/
}
